local Addon = LibStub("AceAddon-3.0"):GetAddon("Langoustine")
local module = Addon:NewModule("ReadyCheck", "AceTimer-3.0", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Langoustine")


module.dataMatrix = {}
module.cells = module.cells or {}
--[[
	Constants
--]]
UNKNOWN=0
READY=1
NOT_READY=2
WAITING=3
DISCONNECTED=4

-- Simple backdrop
local backdrop = {
	bgFile = [[Interface\DialogFrame\UI-DialogBox-Background]],
	edgeFile = [[Interface\DialogFrame\UI-DialogBox-Border]],
	tile = false, edgeSize = 16, tileSize = 16,
	insets = {left = 5, right = 5, top = 5, bottom = 5},
}

function module:OnEnable()
	self:RegisterEvent("READY_CHECK","ReadyCheck")
	self:RegisterEvent("READY_CHECK_CONFIRM","ReadyCheckConfirm")
	self:RegisterEvent("READY_CHECK_FINISHED","ReadyCheckFinished")
	-- if we go into combat hide the ready check window
	self:RegisterEvent("PLAYER_REGEN_DISABLED", function() self:Close() end)
	-- Add a cmdline shortcut
	self:RegisterChatCommand("raready", DoReadyCheck)
end
-- When we are disabled
function module:OnDisable()
	self:Close()
	self:UnregisterAllEvents()
	self:UnregisterChatCommand("raready")
end

function module:ReadyCheckFinished()
	self:ScheduleTimer(function() self:Close() end,2)
end

function module:ReadyCheckConfirm(event, id, confirm)
	if not IsRaidOfficer() then return end
	local name = UnitName("raid"..id)
	local class = select(2,UnitClass("raid"..id))
	if name then
		if confirm == 1 then
			self:MarkUnit(id,name,READY,class)
		elseif confirm == 0 then
			self:MarkUnit(id,name,NOT_READY,class)
		end
	end
end

function module:ReadyCheck(event, officer, duration)
	if not IsRaidOfficer() then return end
	self:Open()
	for i=1,40,1 do
		self:MarkUnit(i,"",0)
	end
	-- scan the raid, and chaneg everyones state
	local raiders = GetNumRaidMembers()
	if raiders > 0 then -- he is in a raid
		for i = 1, raiders, 1 do
			local name, _, _, _, _, class, _, online = GetRaidRosterInfo(i)
			if name then
				if online then
					state = WAITING 
				else 
					state = DISCONNECTED
				end
				if name == officer then
					state = READY
				end
				self:MarkUnit(i,name,state,class)
			end
		end
	end
end
--[[
	Opens the ReadyCheck window.
	this was done as a module originally for my own use, so some
	changes were needed.
--]]
function module:Open()
	local x,y = Addon.db.profile.ReadyCheck.x, Addon.db.profile.ReadyCheck.y
	if not x or not y then
		x = -2
		y = 140
		Addon.db.profile.ReadyCheck.x = x
		Addon.db.profile.ReadyCheck.y = y
	end
	local cellHeight =  15
	local cellWidth = 80
	local title = READY_CHECK
	if frame then 
		frame:SetPoint("CENTER",UIParent,"CENTER",x,y)
		frame:Show()
		return
	end
	frame = CreateFrame("Frame","LangoustineReadyCheckFrame",UIParent)
	table.insert(_G["UISpecialFrames"], "LangoustineReadyCheckFrame") -- close on esc
	frame:SetHeight( (10 * cellHeight) + (cellHeight * 4))
	frame:SetWidth( (4 * cellWidth) + (16 * 5) + 10 )
	frame:SetBackdrop(backdrop)
	frame:SetBackdropColor(0.1, 0.1, 0.1)
	frame:SetBackdropBorderColor(1, 1, 1)
	frame:SetPoint("CENTER",UIParent,"CENTER",x,y)
	frame:RegisterForDrag("LeftButton")
	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:SetFrameStrata("FULLSCREEN_DIALOG")
	-- setup title frame
	frame.title = frame:CreateFontString(nil,"OVERLAY","GameFontWhite")
	frame.title:SetPoint("TOP",frame,"TOP",0,-8)
	frame.title:SetText(title)
	frame:SetClampedToScreen(true)
	frame:SetScript("OnDragStart", function() 
		frame:StartMoving()
	end)
	frame:SetScript("OnDragStop", function(self)
		frame:StopMovingOrSizing()
		local scale,pscale = self:GetEffectiveScale(),self:GetParent():GetEffectiveScale()
		local to,anchor,from,x,y = self:GetPoint()
		local gX,gY = frame:GetLeft() + frame:GetWidth() / 2, frame:GetBottom() + frame:GetHeight() / 2
		local pX,pY = UIParent:GetLeft() + UIParent:GetWidth() / 2, UIParent:GetBottom() + UIParent:GetHeight() / 2
		local x = (gX * scale) - (pX * pscale)
		local y = (gY * scale) - (pY * pscale)
		x = x/scale
		y = y/scale
		frame:ClearAllPoints()
		frame:SetPoint("CENTER",UIParent,"CENTER",x,y)
		Addon.db.profile.ReadyCheck.x = x
		Addon.db.profile.ReadyCheck.y = y
	end)
	local index = 1
	local prevRow = nil
	for row = 1,10 do
		for col = 1,4 do
			local cellIcon = frame:CreateTexture(nil,"OVERLAY")
			local cellText = frame:CreateFontString(nil,"OVERLAY","GameFontNormal")
			cellIcon:SetWidth(16)
			cellIcon:SetHeight(16)
			cellText:SetWidth(cellWidth)
			cellText:SetHeight(cellHeight)
			local cellX = ( (col -1) * -(cellWidth + 16)) - backdrop.insets.left
			local cellY = ( (row -1) * (cellHeight)) - (cellHeight * 5)
			cellIcon:SetPoint("LEFT",frame,"LEFT", -cellX, -cellY )
			cellText:SetPoint("LEFT",cellIcon,"CENTER",8,0)
			cellText:SetJustifyH("LEFT")
			self.cells[index] = {icon= cellIcon, label = cellText}
			index = index + 1
		end
	end
	local closebutton = CreateFrame("Button",nil,frame,"UIPanelButtonTemplate")
	closebutton:SetScript("OnClick", function() 
		module:Close()
	end)
	closebutton:SetPoint("BOTTOM",frame,"BOTTOM",0,10)
	closebutton:SetHeight(20)
	closebutton:SetWidth(100)
	closebutton:SetText("Close")
	frame:Show()
end

--[[
	Mark a given unit
	index is postion index from 1 to 40
	name is the name to set that cell
	state is READY,WAITING,NOT_READY,DISCONNECTED 
--]]
function module:MarkUnit(index,name,state, class)
	if index > 40 or index < 1 then error("Index outside of raid size") end
	if not name then error("name must not be nil") end
	if state < 0 or state > 4 then error("State must be within UNKNOWN to DISCONNECTED") end
	--self.dataMatrix[index] = { ["name"] = name, ["state"] = state}
	local icon = nil
	if state == READY then
		icon = READY_CHECK_READY_TEXTURE
	elseif state == NOT_READY then
		icon = READY_CHECK_NOT_READY_TEXTURE
	elseif state == WAITING then
		icon = READY_CHECK_WAITING_TEXTURE
	elseif state == DISCONNECTED then
		icon = READY_CHECK_AFK_TEXTURE
	end
	self.cells[index].label:SetText(name)
	if class then
		local color = RAID_CLASS_COLORS[class]
		self.cells[index].label:SetTextColor(color.r, color.g, color.b)
	else
		self.cells[index].label:SetTextColor(1,1,1)
	end
	self.cells[index].icon:SetTexture(icon)
end

function module:Close()
	if not frame then return end
	frame:Hide()
end